package njn;

import java.io.*;
import java.util.*;

/**
 *  This class represents a single news article, one whose contents
 *  (head and body) exist on the local machine and are ready for
 *  viewing. 
 *
 *  Initially, articles are represented (in a NewsGroup object) as
 *  ArticleIndexEntry objects in a Vector.  When an article is
 *  downloaded, we instantiate an Article object, which contains
 *  the overview/header data (these fields are inherited from the
 *  superclass), and populate the head and body fields.  The Article
 *  object then replaces the ArticleIndexEntry object (which you can 
 *  think of it as a "stub" article) in the Vector.
 * 
 *  Currently, once an article has been downloaded (or retrieved from
 *  a previous session's cache?), its place in the article vector will 
 *  be as a full Article object (whereas articles which have not been 
 *  opened in the current session will have ArticleIndexEntry objects).  
 * 
 *  However, if accumulated Article objects prove a burden on resources,
 *  I might want to consider destroying the Article object once its 
 *  contents are no longer being actively displayed.  In this case, 
 *  there would only ever be one Article object in existence at a time - 
 *  the one currently being read by the user.
 *
 ****/
public class Article extends ArticleIndexEntry
{
/*
    public String number,
                  subject,
                  author,
                  datestamp,
                  message_id,
                  references,
                  byte_count,
                  line_count;
*/
                          /** Any cross-refererences the article shows. */
    public Vector xrefs; 

    public String head,
                  body;

    // ------------------------------------------------------ constructor
      /**
       *  Takes the name of an ascii file containing the article text.
       *  The file is assumed to contain both head and body, separated
       *  by a blank line (as according to RFC 850).
       *  
       *  The constructor attempts to assign the public variables (above)
       *  by parsing the header for the necessary data.
       *
       ****/
    public Article( String src_file ) {
                                     // implicit call to super() here
        readArticle( src_file );
        parseHeader();
    }

      /**
       * @param article_stub The ArticleIndexEntry object which 
       *  references the article in the article list.  This object is 
       *  already assumed to contain the overview data (number, subject,
       *  author, etc.).
       *
       ****/
    public Article( String src_file, ArticleIndexEntry article_stub ) {
                                     // implicit call to super() here
        number = article_stub.number;
        subject = article_stub.subject;
        author = article_stub.author;
        datestamp = article_stub.datestamp;
        message_id = article_stub.message_id;
        references = article_stub.references;
        byte_count = article_stub.byte_count;
        line_count = article_stub.line_count;
        isOpen = true;

        readArticle( src_file );
    }

      /**
       *  Takes the name of an ascii file containing the article text.
       *  The file is assumed to contain both head and body, separated
       *  by a blank line (as according to RFC 850).
       *
       ****/
    public Article( String src_file, 
                    String num, String subj, String from, String date,
                    String id, String refs, String bytes, String lines )
    {
                                     // implicit call to super() here
        number = num;
        subject = subj;
        author = from;
        datestamp = date;
        message_id = id;
        references = refs;
        byte_count = bytes;
        line_count = lines;
        isOpen = true;

        readArticle( src_file );

    } // ----------------------------------------------------------------


    // ------------------------------------------------------ readArticle
      /*
       *  Read the contents of the file into the article's head and body.
       *
       *  No, this method isn't very elegant-looking.  Maybe I'll try to
       *  tidy it up later.
       *
       */
    private void readArticle( String file_name ) {
        BufferedReader article;
        String s = null;
        StringBuffer text;
        boolean eoh, eof;

        text = new StringBuffer();
        try {
            FileReader fr = new FileReader( file_name );
            article = new BufferedReader( fr );
            eoh = false;
            eof = false;

                    /*
                     * First stage: read the file until a blank line 
                     * (or eof) is encountered, and assign the resulting
                     * text as the article header.
                     * 
                     */
            do {
                try {
                    s = article.readLine();
                } catch( IOException any ) { 
                    eof = true; 
                }
                if ( !eof ) {
                    if ( s.equals("")) {
                        eoh = true;
                    } else {
                        text.append( s );
                        text.append("\n");
                    }
                }
            } while ( !eof && !eoh );
            head = text.toString();
            text = new StringBuffer();

                    /*
                     * Second stage: read the remainder of the file
                     * (until eof), and assign the resulting text as
                     * the article body.
                     *
                     */
            while ( !eof ) {
                try {
                    s = article.readLine();
                } catch( IOException any ) {
                    eof = true;
                }
                if ( !eof ) {
                        /*
                         * Check for charstuffed leading "." - if present,
                         * strip off the stuffing.
                         * 
                         */
                    if ( s.startsWith(".."))
                        text.append( s.substring( 1 ));
                    else
                        text.append( s );
                    text.append("\n");
                }
            }
            body = text.toString();

                        // Now close the file.
            try {
                fr.close();
            } catch( IOException IOe ) {;}
        } catch( FileNotFoundException FNFe ) {;}
    } // ----------------------------------------------------------------


    private void parseHeader( ) {
    } // ---------------------------------------------------------------- 

} // ========================[ End of Article ]==========================

