package njn;

import java.util.*;
/**
 *  Represents an article entry in the article list.  Superclass of
 *  Article class, which represents a full article and its contents.
 *  
 *  Pretty much a glorified struct, albeit a self-initializing one.
 *  All the vital fields are public, in the interests of making the
 *  objects as small as possible.  Since we're likely to have hundreds
 *  of these objects around at a time, I didn't want to take up extra
 *  resources defining "get" and "set" methods for everything.
 *  
 ****/
public class ArticleIndexEntry extends Object
{
                // All the fields from the overview header.

                     /** The article's number in the current newsgroup. */
    public String number,

                                        /** The article's subject line. */
                  subject,

                           /** The subject's author (from header line). */
                  author,
                  
                                /** The date/time stamp on the article. */
                  datestamp,

                                   /** The article's unique message-id. */
                  message_id,

                        /** The contents of the references header line. */
                  references,

                                 /** The size of the article, in bytes. */
                  byte_count,

                         /** The length, in lines, of the article body. */
                  line_count,
		  
                              /** Any other (optional) headers provided */
		  others;

            /**
             *  Tells us whether the article has a body - i.e., whether
             *  or not this is an Article object (as opposed to an 
             *  ArticleIndexEntry object).
             *
             ****/
    public boolean isOpen;

    // -------------------------------------------------------------------
      /**
       *  Default constructor - should only be used by the Article class
       *  (which is a subclass of this one) in its own constructor.  Since
       *  Article immediately goes on to assign all the necessary 
       *  variables itself, here they're simply initialized to empty 
       *  values.
       *
       ****/
    public ArticleIndexEntry( ) {
        isOpen = false;
        number = "none";
        author = "none";
        datestamp = "none";
        message_id = "none";
        references = "none";
        byte_count = "0";
        line_count = "0";
        others = "";
    }

      /**
       * @param overview_header The article's XOVER overview header.
       * 
       ****/
    public ArticleIndexEntry( String overview_header ) {
        isOpen = false;
	parseFields( overview_header );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
      /*
       *  Parses the XOVER overview header into the vital fields for the
       *  article: number, subject, sender, datestamp, message_id, 
       *  references, byte_count, and line_count (all public).  Not all
       *  of these are actually used just yet, but they should be included
       *  for completeness.
       *
       *  The XOVER header itself takes the form:
       *  <number>\t<subject>\t<from>\t<date>\t<id>\t<refs>\t<bytes>\t<lines>\t<optional>
       *
       * @param field_list The article's XOVER overview header.
       *
       */
    private void parseFields( String field_list ) {
        StringTokenizer fields = new StringTokenizer( field_list, "\t", true );
   
	number = nextField( fields );
	subject = nextField( fields );   
	author = nextField( fields );   
	datestamp = nextField( fields );   
	message_id = nextField( fields );   
	references = nextField( fields );   
	byte_count = nextField( fields );
	line_count = nextField( fields );   
	if ( fields.hasMoreTokens() )   
            others = nextField( fields );   

    } // -----------------------------------------------------------------


    // --------------------------------------------------------- nextField   
      /*   
       *  Parse the next field from the XOVER-formatted header entry.
       *  I may want to change this into a static method in some utility
       *  class, to keep this class as light-weight as possible.
       *   
       */   
    private String nextField( StringTokenizer tok )    
    {   
        String t,   
               field;   
   
        try {   
            t = tok.nextToken();   
        } catch( NoSuchElementException NSEe ) { return ("<?>"); }   
        if ( t.equals("\t"))   
            field = "<?>";   
        else {   
            field = t;   
            try {   
                tok.nextToken();            // skip past the next "\t".   
            } catch( NoSuchElementException NSEe2 ) {;}   
        }   
   
        return ( field );   
    } // -----------------------------------------------------------------   

} // ====================[ End of ArticleIndexEntry ]=====================

