package njn;

import java.awt.*;
import java.io.*;
import java.util.*;

public class Config
{
    static final String CONFIG_FILE = "njn.cfg";

    
                /*
                 * Defaults for the configuration variables below...
                 *
                 */
                // Personal defaults
    static final String USER = "unknown";
    static final String EMAIL = "unknown";
    static final String SIGNATURE = "";

                // Network defaults
    static final String START_ONLINE = "false";

                // Colour defaults
    static final String MAIN_BACKGROUND = "0xCCCCCC",
                        TEXT_BACKGROUND = "0xFFFFFF",
                        BUTTON_BACKGROUND = "0xCCCCCC";

                // Font defaults
    static final String MAIN_FONT = "Serif-12",
                        MENU_FONT = "Serif-12",
                        INFO_FONT = "SansSerif-12",
                        LIST_FONT = "Monospaced-14",
                        SUBJECT_FONT = "Serif-BOLD-12",
                        ARTICLE_FONT = "Monospaced-14",
                        COMPOSE_FONT = "Monospaced-14";

                // Window-bounds defaults
    static final String MAIN_WINDOW = "0 22 600 450",
                        GROUP_WINDOW = "10 30 600 420",
                        COMPOSE_WINDOW = "20 30 600 400";

                // Misc defaults
    static final String GROUPLIST_VIEW = "subscribed";
    static final String ARTICLELIST_ROWS = "12";
                /*
                 * Customizable configuration variables...
                 *
                 */
                                          /** The personal name of the user. */
    public String user, 
                                  /** 
                                   *  The user's email address, to appear on 
                                   *  all outgoing messages. 
                                   ****/
                  email, 
                              /** The user's signature file (path+filename). */
                  signature;

    public Vector news_servers;
    public String mail_server;
    public boolean start_online;

    public Color main_background,
                 text_background,
                 button_background;

    public Font main_font,
                menu_font,
                info_font,
                list_font,
                subject_font,
                article_font,
                compose_font;

    public int main_window[],
               group_window[],
               compose_window[];

    public int grouplist_view,
               articlelist_rows;

    public Properties properties;

    // ------------------------------------------------------- constructor
    public Config( ) {

        properties = new Properties();
        try {
            readConfiguration( CONFIG_FILE );
        } catch( FileNotFoundException FNFe ) {
            System.out.println("No configuration file found.  Using default parameters.");
        } catch( IOException IOe ) {
            System.out.println("Unable to open configuration file.  Using default parameters.");
        }

        getUserParams();
        news_servers = new Vector();
        getNetworkParams();
        getFontParams();
        getColourParams();
        getWindowParams();
        getMiscParams();

    } // -----------------------------------------------------------------


    // ------------------------------------------------- readConfiguration
      /*
       *  Attempt to open the configuration file and merge its contents
       *  into the property list.
       *
       */
    private void readConfiguration( String filename )
                 throws FileNotFoundException,
                        IOException
    {
        properties.load( 
            new BufferedInputStream( new FileInputStream( filename ))
        );

    } // -----------------------------------------------------------------


    // ----------------------------------------------------- getUserParams
    private void getUserParams( ) {
        user = properties.getProperty("Personal.Name", USER );
        email = properties.getProperty("Personal.Email", EMAIL );
        signature = properties.getProperty("Personal.Signature", SIGNATURE );
    } // -----------------------------------------------------------------

    
    // -------------------------------------------------- getNetworkParams
      /*
       *  Ultimately, if either NNTP or SMTP server is unspecified, 
       *  the user should be prompted for a value when the program 
       *  first loads.
       *
       */
    private void getNetworkParams( ) {
        int i,
            max;

            // Add the specified news server to the "news_servers" Vector.
        String news_string = properties.getProperty("Network.Nntp", "news");
        if ( news_string != null ) {
            StringTokenizer serv_tok = new StringTokenizer( news_string, ",");
            max = serv_tok.countTokens();
            for ( i = 0; i < max; i++ )
                assignNewsServer( serv_tok.nextToken().trim() );
        }

        // Note: NewsReader should ask the user (on startup) for a news 
        // server if the "news_servers" Vector is empty.

            // Assign the mail server.  Only one SMTP server is 
            // supported, so there's no need to break the string up into
            // a Vector.
        assignMailServer( properties.getProperty("Network.Smtp", "mail"));

        String start_string = properties.getProperty("Network.StartOnline", 
                                                     START_ONLINE           );
        start_online = start_string.equalsIgnoreCase("true") ? true : false;

    } // -----------------------------------------------------------------


    // ----------------------------------------------------- getFontParams
      /*
       *  Interpret the font properties.
       *
       *  Uses getProperty and decode (instead of getFont) so that we can
       *  work with String values (which can be written back to the
       *  configuration file).  See comments for getColourParams, below.
       *
       *  Actually, I probably can use getFont, since keeping the String
       *  looks redundant, now.  (But is this possible?)
       *
       */
    private void getFontParams( ) {
        main_font = Font.decode( properties.getProperty("Fonts.Main", MAIN_FONT ));
        menu_font = Font.decode( properties.getProperty("Fonts.Menu", MENU_FONT ));
        info_font = Font.decode( properties.getProperty("Fonts.Info", INFO_FONT ));
        list_font = Font.decode( properties.getProperty("Fonts.List", LIST_FONT ));
        subject_font = Font.decode( properties.getProperty("Fonts.Subject", 
                                                            SUBJECT_FONT    ));
        article_font = Font.decode( properties.getProperty("Fonts.Article",
                                                            ARTICLE_FONT    ));
        compose_font = Font.decode( properties.getProperty("Fonts.Posting",
                                                            COMPOSE_FONT    ));
    } // -----------------------------------------------------------------


    // --------------------------------------------------- getColourParams
      /*
       *  Interpret the colour properties.
       *
       *  The reason this method uses Color.decode(), instead of the 
       *  easier Color.getColor(), is because we need a String value to 
       *  be used as the default in case the property read fails.
       *  
       *  Property.getProperty() with Color.decode() allows this, but 
       *  Color.getColor takes a Color object as default instead.
       *
       *  We want String values because it makes saving the properties back
	   *  to the configuration file ever so much simpler.  (?)
       *
       */
    private void getColourParams( ) {
        try {
            main_background = Color.decode( 
                                  properties.getProperty("Colours.Main",
                                                         MAIN_BACKGROUND ));
        } catch( NumberFormatException NFe ) { 
            main_background = new Color( Integer.decode( MAIN_BACKGROUND ).intValue() );
        }
        try {
            text_background = Color.decode( 
                                  properties.getProperty("Colours.Text",
                                                         TEXT_BACKGROUND ));
        } catch( NumberFormatException NFe ) {
            text_background = new Color( Integer.decode( TEXT_BACKGROUND ).intValue() );
        }
        try {
            button_background = Color.decode(
                                    properties.getProperty("Colours.Button",
                                                           BUTTON_BACKGROUND ));
        } catch( NumberFormatException NFe ) {
            button_background = new Color( Integer.decode( BUTTON_BACKGROUND ).intValue() );
        }
    } // -----------------------------------------------------------------


    // --------------------------------------------------- getWindowParams
    private void getWindowParams( ) {
        int i;
        StringTokenizer bounds_tok;

        String main_string = properties.getProperty("Window.Main", 
                                                    MAIN_WINDOW    );
        main_window = new int[ 4 ];
        bounds_tok = new StringTokenizer( main_string );
        for ( i = 0; i < 4; i++ ) {
            if ( bounds_tok.hasMoreElements() )
                main_window[ i ] = Integer.parseInt( bounds_tok.nextToken() );
            else main_window[ i ] = 50;     // why not?
        }

        String group_string = properties.getProperty("Window.Group",
                                                     GROUP_WINDOW    );
        group_window = new int[ 4 ];
        bounds_tok = new StringTokenizer( group_string );
        for ( i = 0; i < 4; i++ ) {
            if ( bounds_tok.hasMoreElements() )
                group_window[ i ] = Integer.parseInt( bounds_tok.nextToken() );
            else group_window[ i ] = 50; 
        }

        String compose_string = properties.getProperty("Window.Compose",
                                                       COMPOSE_WINDOW    );
        compose_window = new int[ 4 ];
        bounds_tok = new StringTokenizer( compose_string );
        for ( i = 0; i < 4; i++ ) {
            if ( bounds_tok.hasMoreElements() )
                compose_window[ i ] = Integer.parseInt( bounds_tok.nextToken() );
            else compose_window[ i ] = 50; 
        }

    } // -----------------------------------------------------------------


    // ----------------------------------------------------- getMiscParams
    private void getMiscParams( ) {
        String view_string = properties.getProperty("ShowGroups",
                                                    GROUPLIST_VIEW );
        grouplist_view = view_string.equalsIgnoreCase("Subscribed") ?
                         GroupList.SUBSCRIBED : 
                         GroupList.ALL;
        articlelist_rows = Integer.parseInt( 
                properties.getProperty("ArticleListRows", ARTICLELIST_ROWS )
            );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
      /* 
       *  Written in vague hopes of allowing the NNTP port to be set in 
       *  the configuration file (right now the port number, if specified,
       *  is just discarded - NntpConnection always assumes that it's 119).
       *
       */
    private void assignNewsServer( String server ) {
        String server_name,
               server_port;
        StringTokenizer tok = new StringTokenizer( server );

        server_name = tok.nextToken().trim();
        try {
            server_port = tok.nextToken().trim();
        } catch( NoSuchElementException NSEe ) { server_port = "119"; }

        news_servers.addElement( server_name );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
      /* 
       *  Written in vague hopes of allowing the SMTP port to be set in 
       *  the configuration file (right now the port number, if specified,
       *  is just discarded).  Not particularly relevant right now, since
       *  SMTP isn't supported yet anyway.
       *
       */
    private void assignMailServer( String server ) {
        String server_name,
               server_port;
        StringTokenizer tok = new StringTokenizer( server );

        server_name = tok.nextToken().trim();
        try {
            server_port = tok.nextToken().trim();
        } catch( NoSuchElementException NSEe ) { server_port = "25"; }

        mail_server = server_name;

    } // -----------------------------------------------------------------


    // ------------------------------------------------- saveConfiguration
      /**
       *  To be called by the application when appropriate (probably on
       *  program exit).  First updates the Properties object with the 
	   *  current settings, then writes the result to the configuration 
       *  file.
       *
       ****/
    public void saveConfiguration( ) 
                throws IOException
    {
        int i, num_serv;
		StringBuffer serv_buffer;
        String view;

        properties.put("Personal.Name", user );
		properties.put("Personal.Email", email );
		properties.put("Personal.Signature", signature );

        serv_buffer = new StringBuffer();
		num_serv = news_servers.size();
		for ( i = 0; i < num_serv; i++ ) {
            serv_buffer.append( (String)news_servers.elementAt( i ));
			if ( (i+1) < num_serv )
                serv_buffer.append(", ");
	    }
		properties.put("Network.Nntp", serv_buffer.toString() );
    	properties.put("Network.Smtp", mail_server );

        properties.put("Fonts.Main", encodeFont( main_font ));
        properties.put("Fonts.Menu", encodeFont( menu_font ));
        properties.put("Fonts.Info", encodeFont( info_font ));
        properties.put("Fonts.List", encodeFont( list_font ));
        properties.put("Fonts.Subject", encodeFont( subject_font ));
        properties.put("Fonts.Article", encodeFont( article_font ));
        properties.put("Fonts.Posting", encodeFont( compose_font ));

        properties.put("Colours.Main", encodeColor( main_background ));
        properties.put("Colours.Text", encodeColor( text_background ));
        properties.put("Colours.Button", encodeColor( button_background ));

        properties.put("Window.Main", encodeBounds( main_window ));
        properties.put("Window.Group", encodeBounds( group_window ));
        properties.put("Window.Compose", encodeBounds( compose_window ));

        if ( grouplist_view == GroupList.SUBSCRIBED )
            view = "subscribed";
        else view = "all";
        properties.put("Misc.ShowGroups", view );
        properties.put("Misc.ArticleListRows", 
                       Integer.toString( articlelist_rows ));

        properties.save( new FileOutputStream( CONFIG_FILE ), 
                         "njn configuration file."            );

    } // -----------------------------------------------------------------


    // -------------------------------------------------------- encodeFont
      /*
       *  Encodes a Font object as a String-formatted system property
       *  (since Font itself doesn't provide a mechanism for doing so).
       *
       */
    private String encodeFont( Font font ) {
        StringBuffer output,
                     style;

        output = new StringBuffer();
        output.append( font.getFamily() );

        style = new StringBuffer("-");
        if ( ! font.isPlain() ) {
            if ( font.isBold() )
                style.append("BOLD");
            if ( font.isItalic() )
                style.append("ITALIC");
            style.append("-");
        }
        output.append( style );

        output.append( font.getSize() );

        return ( output.toString() );
    } // -----------------------------------------------------------------


    // ------------------------------------------------------- encodeColor
      /*
       *  Same as encodeFont (above), except for Color objects. 
       *  Same reason.
       *
       */
    private String encodeColor( Color colour ) {
        StringBuffer output;
        
        output = new StringBuffer("0x");
        output.append( Integer.toHexString( colour.getRed() ));
        output.append( Integer.toHexString( colour.getGreen() ));
        output.append( Integer.toHexString( colour.getBlue() ));

        return ( output.toString() );
    } // -----------------------------------------------------------------


    // ------------------------------------------------------ encodeBounds
      /*
       *  Encodes a window-bounds array into a String, suitable for 
       *  writing back to the properties file.
       *
       */
    private String encodeBounds( int[] bounds ) {
        StringBuffer output;
        int i;
        
        output = new StringBuffer();
        for ( i = 0; i < bounds.length; i++ ) {
            output.append( bounds[ i ] );
            output.append(" ");
        }

        return ( output.toString() );
    } // -----------------------------------------------------------------

} // =========================[ End of Config ]===========================

