package njn;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 *  The list of newsgroups which appears on the main control panel 
 *  defined in class <i>NewsReader</i>.
 *  
 ****/
public class GroupList extends List
{
    public static final int SUBSCRIBED = 0;
    public static final int ALL = 1;

                        // See NewsReader.java for a description of these.
    Config cfg;
    NewsManager news;
    NntpConnection nntp;

    int current_view;        // The current list view (all or subscribed).

    // -------------------------------------------------------------------
    public GroupList( Config conf,
                      NewsManager n_data,
                      NntpConnection n_conn )
    {
        cfg = conf;
        news = n_data;
        nntp = n_conn;

        current_view = cfg.grouplist_view;
        this.setMultipleMode( false );
        this.setFont( cfg.list_font );
        this.setBackground( cfg.text_background );

        getGroups();

    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
    public String selectedGroup( ) {
        return this.getSelectedItem();
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
    public void getGroups( ) {

        if ( current_view == ALL )
            fillList( news.getAllGroups() );
        else
            fillList( news.getSubscribedGroups() );

    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /*
        ****/
    private void fillList( Enumeration groups ) {
        StringTokenizer grpInfo;
        String groupName,
               range;

        this.setVisible( false );
        if ( this.getItemCount() > 0 )
            this.removeAll();

                        // get the groups from the newsrc file
        while ( groups.hasMoreElements() ) {
            grpInfo = new StringTokenizer( (String)groups.nextElement(), ", ");
            groupName = grpInfo.nextToken();
            try {
                range = grpInfo.nextToken();
            } catch( NoSuchElementException NSEe ) {}

            this.add("  --  " + groupName );

        }
        this.setVisible( true );

    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /*
        ****/
    void refreshList( Enumeration groups ) {
        StringTokenizer grpInfo;
        String groupName,
               range;


        this.setVisible( false );
        if ( this.getItemCount() > 0 )
            this.removeAll();

                  // get the groups from the newsrc file
        while ( groups.hasMoreElements() ) {
            grpInfo = new StringTokenizer( (String)groups.nextElement() );
            groupName = grpInfo.nextToken();
            try {
                range = grpInfo.nextToken();
            } catch( NoSuchElementException NSEe ) {}

            this.add( groupHeader( groupName ) );
        }
        this.setVisible( true );

    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
    private String groupHeader( String name ) {
        StringTokenizer artInfo;
        String header,
               artCount;
        StringBuffer niceHeader;
        int paddingSize,
            i;

        header = nntp.getGroup( name );
        artCount = "-";
        if ( header.startsWith("211")) {
            artInfo = new StringTokenizer( header );
            artInfo.nextToken();                // skip past the 211
            artCount = artInfo.nextToken();
        }
        paddingSize = 6;
        for ( i = 0; i < artCount.length(); i++ )
            paddingSize--;
        niceHeader = new StringBuffer();
        niceHeader.append( artCount );
        for ( i = 0; i < paddingSize; i++ )
            niceHeader.append(" ");
        niceHeader.append( name );

        return ( niceHeader.toString() );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
    public int getView( ) {
        return ( current_view );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
    public void setView( int state ) {
        current_view = state;
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
    public void add( GroupIndexEntry group_object ) {
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
    private void addListeners( ) {
    } // -----------------------------------------------------------------

} // ========================[ End of GroupList ]=========================
