package njn; 
   
import java.awt.*; 
import java.awt.event.*; 
import java.io.*; 
import java.util.*; 
   
public class GroupWindow extends Frame   
{ 
                        // see NewsReader.java for an explanation of these 
    Config cfg; 
    NewsManager news; 
 
    NewsGroup group; 
                                                           // UI variables 
    MenuBar menu_bar; 
    Menu group_menu, 
         article_menu, 
         article_kill_menu, 
         compose_menu; 
    MenuItem group_refresh, 
             group_catchup, 
             group_close, 
             article_read,   
             article_next,   
             article_prev,   
             article_save,   
             article_decode,   
             article_kill_author,   
             article_kill_thread,   
             compose_new,   
             compose_followup,   
             compose_mail;   
    BorderedPanel toolbar;   
    Button b_postNew,   
           b_read,   
           b_decode,   
           b_refresh,   
           b_catchup,   
           b_follow,   
           b_reply,   
           b_save;   
    List articleList,   
         tempList;   
    ArticlePanel message_window;   
    Label b_help;   
    TextField status;   

    // -------------------------------------------------------------------   
    public GroupWindow( Config conf, NewsManager n_data ) {   
        cfg = conf;   
        news = n_data;   
 
        buildInterface();   
        addListeners();   
    } // -----------------------------------------------------------------   
 
 
    // ----------------------------------------------------- openNewsGroup   
       /**  
        *  Opens the newsgroup represented by the object provided.  This   
        *  method must be called before any group operations may be   
        *  performed, preferably right after instatiating the GroupWindow   
        *  object (as the constructor simply creates the interface).   
        *  
        *  This method sets the NewsGroup class object as passed, calls   
        *  the open() method of NewsGroup, and (if online) requests a   
        *  news refresh.  It also sets the window title and returns the   
        *  article list to the top.   
        *  
        *  Note that "opening" a newsgroup means setting it as the    
        *  currently-active group on both client and server, so that all   
        *  subsequent group operations (article requests, etc) refer to    
        *  that group.  This may actually have been done already when
	*  the NewsGroup object was instantiated, but we can't assume
	*  this (we might not have been online then)... doing it again
	*  can't hurt.
        *  
        * @param newsgroup An already-instatiated NewsGroup object,   
        *  representing the newsgroup to be opened.   
        *    
        ****/   
    public void openNewsGroup( NewsGroup newsgroup ) {   
        String msg;   
        boolean connected;   
   
        group = newsgroup;   
   
        connected = group.open();   
        if ( connected ) {   
            msg = refreshNews();   
            articleList.validate();   
        } else   
            msg = "Group \"" + group.name + "\" open.  Not online.";   
        status.setText( msg );   
        this.setTitle( group.name );   
   
        if ( articleList.getItemCount() > 0 )    
            articleList.select( 0 );   
   
    } // -----------------------------------------------------------------   
   
   
    // -------------------------------------------------------------------   
    public void closeNewsGroup( ) {   
        articleList.removeAll();   
        message_window.clear();   
        group.close();   
        group = null;   
    } // -----------------------------------------------------------------   
   
   
    // -------------------------------------------------------------------   
    public void startWaiting() {   
        this.setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ));   
        message_window.setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ));   
    } // -----------------------------------------------------------------   
   
   
    // -------------------------------------------------------------------   
    public void stopWaiting() {   
        this.setCursor( Cursor.getDefaultCursor() );   
        message_window.setCursor( Cursor.getDefaultCursor() );   
    } // -----------------------------------------------------------------   
   
   
    // -------------------------------------------------------------------   
    private String refreshNews( ) {   
        String mesg;   
   
        status.setText("Stand by: updating newsgroup...");   
        startWaiting();   
   
        mesg = group.refresh();   
        fillArticleList();   
   
        stopWaiting();   
   
        return ( mesg );   
    } // -----------------------------------------------------------------   
   
   
    // --------------------------------------------------- fillArticleList   
    private void fillArticleList( ) { 
     	ArticleIndexEntry current_article;
        String 
	/*
	       header,   
               number,   
               id,   
               refs,
               bytes,
        */   
               subject,   
               author,   
               lines,
               date,
        /*
               other,
	*/   
               pad1, pad2, pad3,
               pretty;
        StringTokenizer header_fields;
        int i;

        articleList.setVisible( false );

        for ( i = 0; i < group.articles.size(); i++ ) {
		current_article = (ArticleIndexEntry)group.articles.elementAt( i );

/*			// This part has been moved to ArticleIndexEntry
            other = "";   
            header = (String)group.headers.elementAt( i ); 
            header_fields = new StringTokenizer( header, "\t", true );
   
            number = nextField( header_fields );           // (not needed)
            subject = nextField( header_fields );   
            author = nextField( header_fields );   
            date = nextField( header_fields );   
            id = nextField( header_fields );   
            refs = nextField( header_fields );   
            bytes = nextField( header_fields );            // (not needed)
            lines = nextField( header_fields );   
            if ( header_fields.hasMoreTokens() )   
                other = nextField( header_fields );
*/   
                           // now make sure the list columns line up nicely.
            pad1 = padColumn( 9, current_article.line_count );
            if ( pad1.equals("..  "))
                lines = current_article.line_count.substring( 0, 5 );
            else lines = current_article.line_count;
            pad2 = padColumn( 54, current_article.subject );
            if ( pad2.equals("..  "))
                subject = current_article.subject.substring( 0, 50 );
            else subject = current_article.subject;
            pad3 = padColumn( 34, current_article.author );
            if ( pad3.equals("..  "))
                author = current_article.author.substring( 0, 30 );
            else author = current_article.author;

                // I should probably reformat the date at some point...
            date = current_article.datestamp;
   
            pretty = new String( lines   + pad1 +
                                 subject + pad2 +
                                 author  + pad3 +
                                 date             );
            articleList.addItem( pretty );
        }
        articleList.setVisible( true );

    } // -----------------------------------------------------------------   
   
   
    // --------------------------------------------------------- padColumn   
      /*   
       *  Help make sure the fields in the list entry line up properly   
       *  (since List doesn't allow columns, we have to visually simulate   
       *  them).  
       *   
       */   
    private String padColumn( int column_size, String field ) {   
        int x,   
            pad_size;   
        StringBuffer padding;   
   
        padding = new StringBuffer();   
        pad_size = column_size - field.length();   
        if ( pad_size < 4 ) {   
            padding.append("..  ");   
        }   
        else {   
            for ( x = 0; x < pad_size; x++ )   
                padding.append(' ');   
        }   
   
        return ( padding.toString() );   
    } // -----------------------------------------------------------------   
   
   
    // --------------------------------------------------------- nextField   
      /*   
       *  Parse the next field from the XOVER-formatted header entry.   
       *   
       */   
    private String nextField( StringTokenizer tok )    
    {   
        String t,   
               field;   
   
        try {   
            t = tok.nextToken();   
        } catch( NoSuchElementException NSEe ) { return ("<?>"); }   
        if ( t.equals("\t"))   
            field = "<?>";   
        else {   
            field = t;   
            try {   
                tok.nextToken();            // skip past the next "\t".   
            } catch( NoSuchElementException NSEe2 ) {;}   
        }   
   
        return ( field );   
    } // -----------------------------------------------------------------   
   
   
    // -------------------------------------------------------------------   
       /** 
        *  Download the selected article from the server and display it.   
        *     
        ****/   
    void getArticle( int index ) {   
        String response,   
               tfile;   
   
        if ( index >= 0 ) {   
            startWaiting();   
            response = group.retrieve( index );   
   
            if ( NntpConnection.checkResponseCode( response,    
                                                   NntpConnection.NNTP_ARTICLE_FOLLOWS ))   
            {   
                tfile = group.article_file;   // will be more elaborate later 
                message_window.displayArticle( tfile );   
            }   
            status.setText( response );   
            stopWaiting();   
        }   
    } // -----------------------------------------------------------------   
   
   
    // -------------------------------------------------------------------   
       /**   
        *  Do not download the selected article, but display it if it    
        *  exists in the cache (not yet implemented).  Also set the   
        *  title and author display on the message pane.   
        *     
        ****/   
    void displayArticle( int index ) {
        ArticleIndexEntry article;
   
        article = (ArticleIndexEntry)group.articles.elementAt( index );
        
        /* 
         * Should check to see if isOpen is true... if it is, will cast to
         * an Article and display the text.
         */

        message_window.setHeading( article.author, article.subject );   

    } // -----------------------------------------------------------------   


    // -------------------------------------------------------------------   
    private void addListeners() {   

        articleList.addActionListener(   
            new ActionListener( ) {   
                public void actionPerformed( ActionEvent e ) {   
                    getArticle( articleList.getSelectedIndex() );   
                }   
            }   
        );   

        articleList.addItemListener(   
            new ItemListener( ) {   
                public void itemStateChanged( ItemEvent e ) {   
                    displayArticle( articleList.getSelectedIndex() );   
                }   
            }   
        );   

        b_read.addActionListener(   
            new ActionListener( ) {   
                public void actionPerformed( ActionEvent e ) {   
                    getArticle( articleList.getSelectedIndex() );   
                }   
            }   
        );   
    } // -----------------------------------------------------------------   
   
   
    // -------------------------------------------------------------------   
    private void buildInterface( ) {   
        Panel mainUI;   
   
        this.setBackground( cfg.main_background );   
        this.setFont( cfg.main_font );   
        buildMenuBar();   
        buildToolbar();   
   
        articleList = new List( cfg.articlelist_rows );   
        articleList.setMultipleMode( false );   
        articleList.setBackground( cfg.text_background );   
        articleList.setForeground( Color.black );   
        articleList.setFont( cfg.list_font );   
   
        message_window = new ArticlePanel( cfg, news );   
   
        status = new TextField();   
        status.setEditable( false );   
        status.setFont( cfg.info_font );   

        mainUI = new Panel( new BorderLayout( 2, 3 ));   
        mainUI.add("North", articleList );   
        mainUI.add("Center", message_window );   

        this.setLayout( new BorderLayout() );   
        this.add("North", toolbar );   
        this.add("Center", mainUI );   
        this.add("South", status );   

        this.setSize( 550, 450 );   
    } // -----------------------------------------------------------------   


    // ------------------------------------------------------ buildMenuBar   
    private void buildMenuBar( ) {   

        group_menu = new Menu("Group");   
        article_menu = new Menu("Article");   
        compose_menu = new Menu("Compose");   

        group_refresh = new MenuItem("Re-check");   
        group_catchup = new MenuItem("Catch-up");   
        group_close = new MenuItem("Close group");   
        group_menu.add( group_refresh );   
        group_menu.add( group_catchup );   
        group_menu.add( group_close );   

        article_read = new MenuItem("Read selected");   
        article_next = new MenuItem("Read next");   
        article_prev = new MenuItem("Read previous");   
        article_save = new MenuItem("Save");   
        article_decode = new MenuItem("Decode");   
        article_kill_menu = new Menu("Kill");   
        article_kill_author = new MenuItem("Author");   
        article_kill_thread = new MenuItem("Thread");   
        article_kill_menu.add( article_kill_author );   
        article_kill_menu.add( article_kill_thread );   
        article_menu.add( article_read );   
        article_menu.add( article_next );  
        article_menu.add( article_prev );   
        article_menu.add( article_save );   
        article_menu.add( article_decode );   
        article_menu.add( article_kill_menu );   

        compose_new = new MenuItem("New article");   
        compose_followup = new MenuItem("Follow-up");   
        compose_mail = new MenuItem("Private reply");   
        compose_menu.add( compose_new );   
        compose_menu.add( compose_followup );   
        compose_menu.add( compose_mail );
 
                // Disable menu items I haven't implemented yet...
        group_refresh.setEnabled( false );
        group_catchup.setEnabled( false );
        article_next.setEnabled( false );
        article_prev.setEnabled( false );
        article_save.setEnabled( false );
        article_decode.setEnabled( false );
        article_kill_menu.setEnabled( false );
        compose_menu.setEnabled( false );

        menu_bar = new MenuBar();   
        menu_bar.add( group_menu );   
        menu_bar.add( article_menu );   
//        menu_bar.add( compose_menu );   
        menu_bar.setFont( cfg.menu_font );   
   
        this.setMenuBar( menu_bar );   
   
    } // -----------------------------------------------------------------   
   
       
    // ------------------------------------------------------ buildToolbar   
    private void buildToolbar( ) {   
/*           
        b_postNew = new IconButton("Post a new article.", "post.gif");   
        b_read = new IconButton("Read the currently selected article.", "retrieve.gif");   
        b_refresh = new IconButton("Re-check the news in this group.", "recheck.gif");   
        b_catchup = new IconButton("Catchup (mark all articles as read).", "catchup.gif");   
        b_decode = new IconButton("Decode selected articles.", "decode.gif");   
        b_follow = new IconButton("Decode selected articles.", "followup.gif");   
        b_reply = new IconButton("Reply to selected article.", "mail.gif");   
        b_save = new IconButton("Save selected article.", "save.gif");   
*/   
        b_postNew = new Button("Post");   
        b_read = new Button("Read");   
        b_refresh = new Button("Refresh");   
        b_catchup = new Button("Catch-up");   
        b_decode = new Button("Decode");   
        b_follow = new Button("Follow-up");   
        b_reply = new Button("Reply");   
        b_save = new Button("Save");   
   
        b_postNew.setBackground( cfg.button_background );   
        b_read.setBackground( cfg.button_background );   
        b_refresh.setBackground( cfg.button_background );   
        b_catchup.setBackground( cfg.button_background );   
        b_decode.setBackground( cfg.button_background );   
        b_follow.setBackground( cfg.button_background );   
        b_reply.setBackground( cfg.button_background );   
        b_save.setBackground( cfg.button_background );   
   
        toolbar = new BorderedPanel( BorderedPanel.RAISED );   
        toolbar.setLayout( new FlowLayout( FlowLayout.LEFT, 4, 3 ));   

        b_refresh.setEnabled( false );
        b_catchup.setEnabled( false );
        b_decode.setEnabled( false );
        b_postNew.setEnabled( false );
        b_follow.setEnabled( false );
        b_reply.setEnabled( false );
        b_save.setEnabled( false );

        toolbar.add( b_refresh );   
        toolbar.add( b_catchup );   
        toolbar.add( new Label(" "));   
        toolbar.add( b_read );   
        toolbar.add( b_decode );   
        toolbar.add( new Label(" "));   
        toolbar.add( b_postNew );   
        toolbar.add( b_follow );   
        toolbar.add( b_reply );   
        toolbar.add( b_save );   
   
    } // -----------------------------------------------------------------   
   
} // =======================[ End of GroupWindow ]========================   
