package njn;

import java.awt.*;
import java.io.*;
import java.util.*;

/**
 *  Handles all offline news management.  Reads and interprets the
 *  NEWSRC file, and updates group data during every online session.
 ****/
public class NewsManager
{
	public static final String GROUP_FILE = "newsrc";

    Config cfg;
    NntpConnection nntp;

    BufferedReader group_data;
    Vector subsgroups,
           allgroups;

    boolean needNewNewsrc;

    // -------------------------------------------------------------------
    public NewsManager( Config conf,
                        NntpConnection n_client )
    {
                // I'll want to wait until an explicit server connection
        cfg = conf;
        nntp = n_client;
        getGroups();
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
    private String getGroups( ) {
        FileReader group_file;
		String message;

        subsgroups = new Vector();
        allgroups = new Vector();

        needNewNewsrc = false;
		message = "Reading group list...";

        String fileName = new String( GROUP_FILE );
        try {
            group_file = new FileReader( fileName );
        } catch( FileNotFoundException FNFe ) {
            group_file = null;
        }

        if ( group_file == null ) {
            needNewNewsrc = true;
            if ( nntp.connected )
                message = buildNewsrc();
            else
                message = "No newsrc file.";
        } else
            readGroups( group_file );

		return ( message );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /*
        *  Build a new newsrc file.
        *
        */
    private String buildNewsrc( ) {
    	String msg;
        msg = "Retrieving groups list from server";
//        nntp.transmit("LIST");
// need a buildNewsrc method in NntpConnection.
        allgroups.addElement("comp.editors");
        allgroups.addElement("news.answers");
        allgroups.addElement("rec.arts.manga");
        needNewNewsrc = false;
		return ( msg );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /*
        *  Read the list of newsgroups from the newsrc file.
        *
        */
    private void readGroups( FileReader newsrc ) {
        String s_in;
        group_data = new BufferedReader( newsrc );

        s_in = readFileLine();
        while ( s_in != null ) {
            try {
                categorizeGroup( s_in );
            } catch( NoSuchElementException NSEe ) {}
            s_in = readFileLine();
        }

        try {
            group_data.close();
            newsrc.close();
        } catch( IOException IOe ) {}

    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
    private String readFileLine( ) {
        String s;
        try {
            s = group_data.readLine();
        } catch( IOException any ) {
            s = null;
        }
        return s;
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /*
        *  As group is read from newsrc file, place it into the 
		*  appropriate list: subscribed or all.  Subscribed groups have
		*  a : appended to the group name, non-subscribed groups have a !.
        *
        */
    private void categorizeGroup( String grp )
    throws NoSuchElementException
    {
        String g_name, 
               g_type;
        StringTokenizer gtok = new StringTokenizer( grp, "!: ", true );
        g_name = gtok.nextToken();
        g_type = gtok.nextToken();

        if ( g_type.equals(":")) {
            subsgroups.addElement( g_name );
            allgroups.addElement( g_name + "\t*");
        }
        else allgroups.addElement( g_name );
    } // -----------------------------------------------------------------


	// -------------------------------------------------------------------
	  /**
	   *  Change/set a group's status - subscribed or not.
	   *
	   ****/
	public void classifyGroup( String group, boolean subscribe ) {
		if (( subscribe ) && ( ! subsgroups.contains( group )))
			subsgroups.addElement( group );
		else if ( subsgroups.contains( group ))
			subsgroups.removeElement( group );
	} // -----------------------------------------------------------------	


	// -------------------------------------------------------------------
	public void writeNewsrc( ) {
	} // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /**
        *
        ****/
    public Enumeration getSubscribedGroups( ) {
        return ( subsgroups.elements() );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /**
        *
        ****/
    public Enumeration getAllGroups( ) {
        return ( allgroups.elements() );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /**
        *  Return the last date of access for the specified newsgroup
        *  (if available).  Returns "0" if there is no date recorded.
        * @deprecated I'm changing the format of newsrc to get rid
        *  of this field.
        ****/
    public String lastDate( String newsgroup ) {
        String ldate,
               tmp;
        StringTokenizer nameTok;
        boolean found;
        Enumeration groups = getAllGroups();

        tmp = new String();
        ldate = new String();
        found = false;
        try {
            while (( groups.hasMoreElements() ) && ( ! found )) {
                tmp = (String)groups.nextElement();
                if ( tmp.indexOf( newsgroup ) > -1 ) {
                    nameTok = new StringTokenizer( tmp );
                    found = true;
                    nameTok.nextToken();
                    if ( nameTok.hasMoreTokens() ) {
                        ldate = nameTok.nextToken();
                    } else ldate = "0";
                }
            }
            if ( ! found )
                ldate = "0";
        } catch ( NoSuchElementException NSEe ) { ldate = "0"; }

        return ( ldate );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /**
        *  Return the last time of access for the specified newsgroup
        *  (if available).  Returns "0" if there is no date recorded.
        * @deprecated I'm changing the format of newsrc to get rid
        *  of this field.
        ****/
    public String lastTime( String newsgroup ) {
        String ltime,
               tmp;
        StringTokenizer nameTok;
        boolean found;
        Enumeration groups = getAllGroups();

        tmp = new String();
        ltime = new String();
        found = false;
        try {
            while (( groups.hasMoreElements() ) && ( ! found )) {
                tmp = (String)groups.nextElement();
                if ( tmp.indexOf( newsgroup ) > -1 ) {
                    nameTok = new StringTokenizer( tmp );
                    found = true;
                    nameTok.nextToken();
                    if ( nameTok.hasMoreTokens() ) {
                        nameTok.nextToken();
                        ltime = nameTok.nextToken();
                    } else ltime = "0";
                }
            }
            if ( ! found)
                ltime = "0";
        } catch ( NoSuchElementException NSEe ) { ltime = "0"; }

        return ( ltime );
    } // -----------------------------------------------------------------


    // -------------------------------------------------------------------
       /**
        *  Return the article number of the last read article in the 
        *  specified newsgroup (if available).  
        *  Returns "0" if there is none recorded.
        ****/
    public String lastReadArticle( String newsgroup ) {
        String lnumber,
               tmp;
        StringTokenizer nameTok;
        boolean found;
        Enumeration groups = getAllGroups();

        tmp = new String();
        lnumber = new String();
        found = false;
        try {
            while (( groups.hasMoreElements() ) && ( ! found )) {
                tmp = (String)groups.nextElement();
                if ( tmp.indexOf( newsgroup ) > -1 ) {
                    nameTok = new StringTokenizer( tmp );
                    found = true;
                    nameTok.nextToken();
                    if ( nameTok.hasMoreTokens() ) {
                        lnumber = nameTok.nextToken();
                    } else lnumber = "0";
                }
            }
            if ( ! found )
                lnumber = "0";
        } catch ( NoSuchElementException NSEe ) { lnumber = "0"; }

        return ( lnumber );
    } // -----------------------------------------------------------------


/*
move parsing of groupname from enumeration into here?
 
*/
}
